// Copyright (c) 2018 GeometryFactory Sarl  (France).
// All rights reserved.
//
// This file is part of CGAL (www.cgal.org).
//
// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_3/include/CGAL/internal/Mesh_3/get_index.h $
// $Id: get_index.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
// SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
//
// Author(s)     : Laurent Rineau
//
//******************************************************************************
// File Description : trivial backward-compatibility for undocumented
//                    classes/function templates
//******************************************************************************

#ifndef CGAL_INTERNAL_MESH_3_GET_INDEX_3_H
#define CGAL_INTERNAL_MESH_3_GET_INDEX_3_H

#include <CGAL/license/Mesh_3.h>

#include <CGAL/internal/Mesh_3/indices_management.h>

#endif // CGAL_INTERNAL_MESH_3_GET_INDEX_3_H
