/****************************************************************************
 * Core Library Version 1.7, August 2004
 * Copyright (c) 1995-2004 Exact Computation Project
 * All rights reserved.
 *
 * This file is part of CGAL (www.cgal.org).
 *
 * file: GmpIO.cpp
 *                 Adapted from multi-files under /cxx in GMP's source distribution
 *
 * Zilin Du, 2003
 *
 * $URL$
 * $Id$
 * SPDX-License-Identifier: LGPL-3.0-only
 ***************************************************************************/

#ifndef CGAL_HEADER_ONLY

#include <CGAL/CORE/Gmp.h>
#include <CGAL/CORE/Gmp_impl.h>

#endif
