// Copyright (c) 1997  INRIA Sophia-Antipolis (France).
// All rights reserved.
//
// This file is part of CGAL (www.cgal.org).
//
// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Alpha_shapes_3/include/CGAL/Alpha_shape_euclidean_traits_3.h $
// $Id: Alpha_shape_euclidean_traits_3.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
// SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
//
//
// Author(s)     : Tran Kai Frank DA <Frank.Da@sophia.inria.fr>

#ifndef CGAL_ALPHA_SHAPE_EUCLIDEAN_TRAITS_3_H
#define CGAL_ALPHA_SHAPE_EUCLIDEAN_TRAITS_3_H

#include <CGAL/license/Alpha_shapes_3.h>


namespace CGAL {

template <class K>
class Alpha_shape_euclidean_traits_3 : public K {};


} //namespace CGAL

#endif //CGAL_ALPHA_SHAPE_EUCLIDEAN_TRAITS_3_H
