// Copyright (c) 2018 INRIA Sophia-Antipolis (France).
// All rights reserved.
//
// This file is part of CGAL (www.cgal.org).
//
// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Shape_detection/include/CGAL/Regularization.h $
// $Id: Regularization.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
// SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
//
//
// Author(s)     : Dmitry Anisimov
//

#ifndef CGAL_REGULARIZATION_H
#define CGAL_REGULARIZATION_H

#include <CGAL/license/Shape_detection.h>
#include <CGAL/Regularization/regularize_planes.h>

#endif // CGAL_REGULARIZATION_H
