// Copyright (c) 2005-2009  INRIA Sophia-Antipolis (France).
// All rights reserved.
//
// This file is part of CGAL (www.cgal.org)
//
// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/CGAL_ipelets/include/CGAL/CGAL_Ipelet_base.h $
// $Id: CGAL_Ipelet_base.h 0779373 2020-03-26T13:31:46+01:00 Sébastien Loriot
// SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
//
//
// Author(s)     : Sebastien Loriot, Sylvain Pion


#include <CGAL/config.h>

#ifdef CGAL_USE_IPE_7
#include <CGAL/CGAL_Ipelet_base_v7.h>
#else
#include <CGAL/CGAL_Ipelet_base_v6.h>
#endif
