// Copyright (c) 2005-2008  INRIA Sophia-Antipolis (France).
//               2008 GeometryFactory, Sophia Antipolis (France)
// All rights reserved.
//
// This file is part of CGAL (www.cgal.org)
//
// $URL$
// $Id$
// SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
//
// Author(s)     : Laurent Rineau, Pierre Alliez

#ifndef CGAL_HEADER_ONLY

#include <CGAL/Image_3.h>
#include <CGAL/Image_3_impl.h>

#endif // CGAL_HEADER_ONLY
