// Copyright (c) 2010,2012 GeometryFactory Sarl (France).
// All rights reserved.
//
// This file is part of CGAL (www.cgal.org).
//
// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-5.0.3/Mesh_3/include/CGAL/Mesh_3/squared_distance_Point_3_Triangle_3.h $
// $Id: squared_distance_Point_3_Triangle_3.h 254d60f 2019-10-19T15:23:19+02:00 Sébastien Loriot
// SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-Commercial
//
//
// Author(s)     : Laurent Rineau
//

#ifndef CGAL_SQUARED_DISTANCE_POINT_3_TRIANGLE_3_H
#define CGAL_SQUARED_DISTANCE_POINT_3_TRIANGLE_3_H

#include <CGAL/license/Mesh_3.h>


#include <CGAL/squared_distance_3_2.h>

#endif // CGAL_SQUARED_DISTANCE_POINT_3_TRIANGLE_3_H
