// Copyright (c) 2018  GeometryFactory Sarl (France).
// All rights reserved.
//
// This file is part of CGAL (www.cgal.org).
//
// $URL$
// $Id$
// SPDX-License-Identifier: GPL-3.0-or-later
//
// Author(s)     : Maxime Gimeno <maxime.gimeno@geometryfactory.com>

#ifndef CGAL_HEADER_ONLY

#include <CGAL/Qt/frame.h>
#include <CGAL/Qt/frame_impl.h>

#endif // CGAL_HEADER_ONLY
