#ifndef SCENE_POINTS_WITH_NORMAL_ITEM_CONFIG_H
#define SCENE_POINTS_WITH_NORMAL_ITEM_CONFIG_H

#ifdef scene_points_with_normal_item_EXPORTS
#  define SCENE_POINTS_WITH_NORMAL_ITEM_EXPORT Q_DECL_EXPORT
#else
#  define SCENE_POINTS_WITH_NORMAL_ITEM_EXPORT Q_DECL_IMPORT
#endif

#endif // SCENE_POINTS_WITH_NORMAL_ITEM_CONFIG_H
